/* alternatives subsystem
(c) 2003
Stanislav Ievlev <inger@altlinux.org>
*/

#ifndef ALT_STORAGE_H
#define ALT_STORAGE_H

#include <map>
#include <string>

#include <libing/config.h>
#include <libing/file_lock.h>
#include <libing/helpers.h>

#include <alternatives/alternative.h>

using Ing::unslash;

/**
 * General alternatives subsystem representation,
 * Using this class you can change any alternatives settings. However,
 * you also need alternatives-update routine to recreate symlinks
 */
class AltStorage: public Ing::Config
{
public:
	/**
	 * A constructor
	 * setup general internal structures, made database locking
	 * @param currdir directory where all alternatives configs live
	 * @param _mode locking mode, default lockfile name is hard compilled
	 */
	AltStorage(const std::string& currdir, Ing::FileLock::access_mode _mode);
	/**
	 * A destructor
	 * unlock database, delete all internal structures, dump static information into /etc/alternatives/auto/dump.xml
	 * @see dump()
	 */
	virtual ~AltStorage();
	/**
	 * subconstructor, parse all alternatives configuration files, create internal structures,
	 * made consistency checkings
	 * @see check()
	 * @throw Ing::internal_error
	 */
	void	parse();
	/**
	 * change settings of one alternatives item
	 * we try to replace item, then made consistency checking and rollback if it's fail.
	 * @param old_alt previos alternative item
	 * @param new_alt updated alternative item
	 * @return true on success, otherwise false 
	 * @see check()
	 */
	bool	setup(Alternative *old_alt, Alternative *new_alt);
	/**
	 * delete candidate and all it's slave candidates from alternatives structures
	 * @param name name of candidate
	 */
	void	ignore(const char *name);

	alternative_list	alternatives; /**< list of all possible alternatives */
	alternative_list	candidates; /**< all possible candidates list and it's alternatives */
protected:
	/**
	 * implementation of Ing::Config method. 
	 * Creates new Candidates objects, setup general internal configurations
	 * @param group name of the group
	 */
	virtual	void	begin_group(const std::string& group);
	/**
	 * implementation of Ing::Config method.
	 * Creates or update existing Alternatives object, fill object with data, also setup slaves
	 * if exists
	 * @param group name of the group
	 */
	virtual	void	end_group(const std::string& group);
	/**
	 * implementation of Ing::Config method.
	 * Check are paths and weights of the alternative items correct,
	 * setup some internal data structures
	 * @param group name of the group
	 * @param name name of the option
	 * @param value value of the option
	 * @param type type of the option
	 * @throw Ing::internal_error
	 */
	virtual	void	setup_option(const std::string& group, const std::string& name,
				  const std::string& value, const std::string& type);
private:
	/**
	 * Fill internal biffer with xml from files under selected directory
	 * @param path path to the directory with configuration files
	 * @see buffer
	 */
	std::string fill_from(const std::string& path);
	/**
	 * Made a consistency checking in all available alternatives
	 * @param during_test in non testing mode alternative item will 
	 * be reset into auto mode on problems
	 * @return empty string on success, otherwise problem description
	 */
	std::string	check(bool during_test = false);
	/**
	 * setup all slave items for the current alternative, founded during previos setup_option()
	 * functions.
	 * Check for consistency,
	 * setup as new slave alternative item with candidates
	 * @param candidate_name real name of master candidate, we setup now in end_group()
	 * @param curr_item current master alternative
	 * @throw Ing::internal_error
	 * @see setup_option
	 * @see end_group()
	 */
	void	setup_slaves(const std::string& candidate_name, Alternative *curr_item);
	/**
	 * Dump all alternatives in manual mode into special dump.xml file
	 * @throw Ing::internal_error
	 */
	void	dump(); //dump manual settings
	/**
	 * Helper function, checks are this pointer is null, raise exception otherwise
	 * @param pointer pointer for check
	 * @param errstr error string for exception
	 * @throw Ing::internal_error
	 */
	void	check_exists(void *pointer, const std::string& errstr);
	/**
	 * Helper function, checks are this pointer is false, raise exception otherwise
	 * @param pointer to check
	 * @param errstr error string for exception
	 * @throw Ing::internal_error
	 */
	void	check_exists(bool pointer, const std::string& errstr);
	/**
	 * Helper function, checks are path valid alternatives link name.
	 * Alternatives link name must begin with slash
	 * @param str link path
	 * @throw Ing::internal_error
	 */
	std::string	prepare_path(const std::string& str);
	/**
	 * Helper function, replace some symbols with entities (e.g. '&' with amp;),
	 * for safe using with xml
	 * @param str string to convert
	 * @return string ready to place into xml file
	 */
	static std::string	xmlsafe(std::string str);

	Candidate	*curr_prop; /**< current candidate item */
	Ing::FileLock::access_mode	mode; /**< locking mode */
	std::string	current_dir; /**< working directory with alternatives config files */
	Ing::FileLock	locker; /**< File Locker */
};

#endif
